"""
This file contains exceptions, raised by forms.* modules
"""
from typing import List

from django.core.exceptions import ValidationError


class FileSizeError(ValidationError):
    def __init__(self, allowed_size, file_size):  # type: (int, int) -> None
        error = "File is too big, max %d bytes are allowed (it has %d bytes)." % (allowed_size, file_size)
        super(FileSizeError, self).__init__(error, code=413)


class FileTypeError(ValidationError):
    def __init__(self, extension, valid_extensions):  # type: (str, List[str]) -> None
        error = "File has invalid type, only [%s] are allowed (it has '%s')."  \
                % (', '.join(valid_extensions), extension)
        super(FileTypeError, self).__init__(error, code=422)



from setuptools import setup

with open("README.md", "r") as fh:
    long_description = fh.read()

requires = []
with open('requirements.txt') as f:
    for line in f.readlines():
        line = line.strip()  # Remove spaces
        line = line.split('#')[0]  # Remove comments
        if line:  # Remove empty lines
            requires.append(line)

setup(
    name='django-rest-form-fields',
    version='1.3.1',
    packages=['django_rest_form_fields'],
    package_dir={'': 'src'},
    url='https://github.com/M1hacka/django-rest-form-fields',
    license='BSD 3-clause "New" or "Revised" License',
    author='Mikhail Shvein',
    author_email='work_shvein_mihail@mail.ru',
    description='Extended form fields to validate REST-request data via django.forms',
    long_description=long_description,
    long_description_content_type="text/markdown",
    requires=requires
)



"""
This file contains extended BaseForm class
"""
from django import forms
from typing import Any


class SourceFormMixin(object):
    """
    Adds hooks that need to be called in order to process field's source attribute
    """
    def _post_init(self, *args, **kwargs):  # type: (*Any, **Any) -> None
        """
        Replace attribute names with source names if they exist and remember replaces
        :param args: Init arguments
        :param kwargs: Init kwargs
        :return: None
        """
        self._src_replaces = {}
        result_fields = {}
        for name, f in self.fields.items():
            if getattr(f, 'source', None):
                self._src_replaces[f.source] = name
                result_fields[f.source] = f
            else:
                result_fields[name] = f

        self.fields = result_fields

    def _post_clean_fields(self):  # type: () -> None
        """
        Replace source field names with attribute names
        :return: None
        """
        self.cleaned_data = {self._src_replaces.get(name, name): val for name, val in self.cleaned_data.items()}


class BaseForm(forms.Form, SourceFormMixin):
    def __init__(self, *args, **kwargs):
        super(BaseForm, self).__init__(*args, **kwargs)
        self._post_init(*args, **kwargs)

    def _clean_fields(self):
        super(BaseForm, self)._clean_fields()
        self._post_clean_fields()


class BaseModelForm(forms.ModelForm, SourceFormMixin):
    def __init__(self, *args, **kwargs):
        super(BaseModelForm, self).__init__(*args, **kwargs)
        self._post_init(*args, **kwargs)

    def _clean_fields(self):
        super(BaseModelForm, self)._clean_fields()
        self._post_clean_fields()
